/**
 * @file
 * 
 * @author Petr Bravenec <petr.bravenec@hobrasoft.cz>
 * @copyright MIT License
 */
#include "html5stream.h"
#include <QDebug>
#include <QRegExp>
#if QT_VERSION > 0x050000
#include <QJsonObject>
#include <QJsonParseError>
#else
#ifdef WIN32
#include <QJson/Parser>
#include <QJson/Serializer>
#else
#include <qjson/parser.h>
#include <qjson/serializer.h>
#endif
#endif


Html5Stream::Html5Stream(QObject *parent) : QObject(parent) {
    m_manager = new QNetworkAccessManager(this);
    connect (m_manager, SIGNAL(finished(QNetworkReply *)),
             this,        SLOT(finished(QNetworkReply *)));
}


QNetworkReply *Html5Stream::get (const QNetworkRequest& request) {
    return makeReply(m_manager->get(request));
}


QNetworkReply *Html5Stream::post(const QNetworkRequest& request, QIODevice *device) {
    return (m_manager->post(request, device));
}


QNetworkReply *Html5Stream::post(const QNetworkRequest& request, QHttpMultiPart *multipart) {
    return (m_manager->post(request, multipart));
}


QNetworkReply *Html5Stream::post(const QNetworkRequest& request, const QByteArray& data) {
    return (m_manager->post(request, data));
}


QNetworkReply *Html5Stream::makeReply(QNetworkReply *reply) {
    connect(reply, SIGNAL(readyRead()), this, SLOT(dataArrived()));
    return reply;
}

void Html5Stream::dataArrived() {
    QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
    if (reply == NULL) { return; }
    QString event;
    while (reply->canReadLine()) {
        QByteArray line = reply->readLine();
        if (line.startsWith("event:")) {
            event = QString::fromUtf8(line).replace(QRegExp("^event:\\s*"),"").trimmed();
            continue;
            }
        if (line.startsWith("data")) {
            QString textdata = QString::fromUtf8(line).replace(QRegExp("^data:\\s*"), "").trimmed();
            emit data(event, textdata);
            emitJsonData(event, textdata);
            continue;
            }
        }
}

void Html5Stream::finished(QNetworkReply *reply) {
    QNetworkRequest request = reply->request();
    switch(reply->operation()) {
        case QNetworkAccessManager::GetOperation:
            makeReply(m_manager->get(request));
            break;
        default:
            qDebug() << "Automatic reconnection not suppported for this http method";
            break;
        }
    reply->deleteLater();
}


void Html5Stream::emitJsonData(const QString& event, const QString& textdata) {
    if (receivers(SIGNAL(data(const QString&, const QVariant&))) <= 0) {
        return;
        }
    #if QT_VERSION > 0x050000
    QJsonParseError error;
    QJsonDocument jdoc = QJsonDocument::fromJson(textdata.toUtf8(), &error);
    emit data(event, jdoc.toVariant());
    #else
    bool ok;
    QJson::Parser parser;
    emit data(event, parser.parse(textdata.toUtf8(), &ok));
    #endif
}



