/**
 * @file
 * 
 * @author Petr Bravenec <petr.bravenec@hobrasoft.cz>
 * @copyrigt MIT License
 */
#ifndef _Html5Stream_H_
#define _Html5Stream_H_

#include <QObject>
#include <QNetworkAccessManager>
#include <QNetworkReply>

/**
 * @brief Napojení na HTML5 proud událostí
 *
 * Třída se napojí na HTML5 proud událostí a každou událost hlásí pomocí signálů.
 * Třída posílá dva stejné signály, ovšem s jinými datovými typy:
 * - QString - text události
 * - QVariant - rozparsovaný JSON poslaný v události
 *
 * Ke třídě je možné připojit najednou několik proudů událostí, rozlišit je je možné
 * podle identifikátoru streamu v parametru event signálu data(). Pozor: identifikátor 
 * streamu je nastavený na straně serveru a klientská část jej nemá jak ovlivnit.
 * Při připojení více proudů událostní najednou nemusí být možné rozeznat
 * mezi jednotlivými zdroji událostí.
 *
 * Před odesláním události typu QVariant se testuje, jestli je na signál někdo připojený.
 * Pokud není nikdo připojený, json data se neparsují a signál se neposílá.
 *
 * Pro jednoduché používání jsou implementované metody get() a post() odpovídající svým protejškům třídy QNetworkAccessManager.
 * Pokud je potřeba nastavit další vlastnosti použitého QNetworkAccessManager, je možné využít metodu networkAccessManager().
 *
 */
class Html5Stream : public QObject {
    Q_OBJECT
  public:
    Html5Stream(QObject *);

    QNetworkReply *get (const QNetworkRequest&);
    QNetworkReply *post(const QNetworkRequest&, QIODevice *);
    QNetworkReply *post(const QNetworkRequest&, QHttpMultiPart *);
    QNetworkReply *post(const QNetworkRequest&, const QByteArray&);

    /**
     * @brief Vrací odkaz na použitý QNetworkAccessManager
     */
    QNetworkAccessManager *networkAccessManager() const { return m_manager; }

  signals:
    /**
     * @brief Předává surová, nezpracovaná data
     * 
     * V parametru event je předaný název události.
     */
    void data(const QString& event, const QString&);


    /**
     * @brief Předává surová, nezpracovaná data
     * 
     * V parametru event je předaný název události.
     */
    void data(const QString& event, const QVariant&);

  private slots:
    void dataArrived();
    void finished(QNetworkReply *);

  private:
    QNetworkReply          *makeReply(QNetworkReply *);
    QNetworkAccessManager  *m_manager;
    void                    emitJsonData(const QString& event, const QString& data);

};

#endif

